@@ -1,5 +1,3 @@
-# $Id: proxy.py,v 1.20 2011/05/26 20:19:17 edwlan Exp $
-
 #
 #  Copyright (c) 2011 Edward Langley
 #  All rights reserved.
@@ -33,7 +31,8 @@
 #
 #
 import copy
-import urllib
+import cookielib
+import urllib2
 import urlparse
 import itertools
 import traceback
@@ -74,15 +73,14 @@ class ProxyEvents(object):
 	'''An event handler for JSONRPCProxy'''
 
 	#: an instance of a class which defines a __get__ method, used to generate a request id
-	IDGen = IDGen
+	IDGen = IDGen()
 
 
 	def __init__(self, proxy):
 		'''Allow a subclass to do its own initialization, gets any arguments leftover from __init__'''
-		self.IDGen = self.IDGen()
 		self.proxy = proxy
 
-	def get_postdata(self, args, kwargs):
+	def get_params(self, args, kwargs):
 		'''allow a subclass to modify the method's arguments
 
 		e.g. if an authentication token is necessary, the subclass can automatically insert it into every call'''
@@ -108,14 +106,15 @@ class JSONRPCProxy(object):
 	_eventhandler = ProxyEvents
 	def customize(self, eventhandler):
 		self._eventhandler = eventhandler(self)
+		return self
 
 	def _transformURL(self, serviceURL, path):
-		if serviceURL[-1] == '/':
+		if serviceURL.endswith('/'):
 			serviceURL = serviceURL[:-1]
-		if path[0] != '/':
-			path = '/%s'%path
-		if path[-1] != '/' and '?' not in path:
-			path = '%s/'%path
+		if path.endswith('/'):
+			path = path[:-1]
+		if path.startswith('/'):
+			path = path[1:]
 		return serviceURL, path
 
 
@@ -131,33 +130,50 @@ class JSONRPCProxy(object):
 		return cls(url, path, serviceName, ctxid)
 
 
-	def __init__(self, host, path='/jsonrpc', serviceName=None, *args, **kwargs):
+	def __init__(self, host, path='jsonrpc', serviceName=None, *args, **kwargs):
 		self.serviceURL = host
 		self._serviceName = serviceName
 		self._path = path
 		self.serviceURL, self._path = self._transformURL(host, path)
 		self.customize(self._eventhandler)
 
+		cj = cookielib.CookieJar()
+		self._opener = urllib2.build_opener(urllib2.HTTPCookieProcessor(cj))
 
 
+
+	def _set_opener(self, opener):
+		self._opener  = opener
+		return self
+
 	def __getattr__(self, name):
 		if self._serviceName != None:
 			name = "%s.%s" % (self._serviceName, name)
-		return self.__class__(self.serviceURL, path=self._path, serviceName=name)
+		return self.__class__(self.serviceURL, path=self._path, serviceName=name).customize(type(self._eventhandler))._set_opener(self._opener)
 
 
 	def _get_postdata(self, args=None, kwargs=None):
-		args,kwargs = self._eventhandler.get_postdata(args, kwargs)
+		_args, _kwargs = self._eventhandler.get_params(args, kwargs)
 		id = self._eventhandler.IDGen
-		result = Request(id, self._serviceName, args, kwargs)
+		result = Request(id, self._serviceName, _args, _kwargs)
 		return jsonrpc.jsonutil.encode(result)
 
+	def _get_url(self):
+		result = [self.serviceURL]
+		if self._path:
+			result.append(self._path)
+		#result.append('')
+		return '/'.join(result)
+
+	def _post(self, url, data):
+		return self._opener.open(url, data)
 
 	def __call__(self, *args, **kwargs):
 
-		url = '%(host)s%(path)s' % dict(host = self.serviceURL, path = self._path)
+		url = self._get_url()
 		postdata = self._get_postdata(args, kwargs)
-		respdata = urllib.urlopen(url, postdata).read()
+		#respdata = urllib2.urlopen(url, postdata).read()
+		respdata = self._post(url, postdata).read()
 		resp = Response.from_dict(jsonrpc.jsonutil.decode(respdata))
 		resp = self._eventhandler.proc_response(resp)
 
@@ -184,7 +200,7 @@ class JSONRPCProxy(object):
 		if hasattr(methods, 'items'): methods = methods.items()
 		data = [ getattr(self, k)._get_postdata(*v) for k, v in methods ]
 		postdata = '[%s]' % ','.join(data)
-		respdata = urllib.urlopen(self.serviceURL, postdata).read()
+		respdata = self._post(self._get_url(), postdata).read()
 		resp = Response.from_json(respdata)
 		try:
 			result = resp.get_result()
